// BasicTypes.h: interface for the BasicTypes class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __BASICTYPES_H__
#define __BASICTYPES_H__

#include <sstream>

#include <meta/Type.h>
#include <meta/Value.h>

class BasicTypeValue : public Value
{
public:
	static Type getTypeId() {
		if ( mBasicTypeId.isBad() ) {
			mBasicTypeId = Type::registerAbstractType(
				Value::getTypeId(),"basic");
		}
		return mBasicTypeId;
	}
protected:
//	BasicTypeValue() {}
private:
	static Type mBasicTypeId;

};

class URI : public BasicTypeValue
{
public:
	URI(const char *str) 
	{ mValue = str; }
	URI(const URI& iv)
	{	mValue = iv.mValue;	}
	
	static Type getTypeId() {
		if ( mURITypeId.isBad() ) {
			mURITypeId = Type::registerType(
				BasicTypeValue::getTypeId(),"uri",
				createStringValue);
		}
		return mURITypeId;
	}
	
	static IValue* createStringValue(const char* str, void*) { return new URI(str); }
	
	Type getType() const { return URI::getTypeId(); }
	IValue* clone() const { return new URI(*this); }

	// --- String representation support
	// get the string representation of the value
	// this is usually shorter representation of the value
	// (for eg. string -> shorter version of the complete string, uri)
	const char* getStringRepresentation() const
	{	return mValue.c_str();		}
	// get the actual contents of the value
	//  eg. 1. uri -> will give the contents of the file.
	//      2. string -> will give the complete string
	// NOTE the difference from getStringRepresentation()
	const char* getStringContents() const
	{	return getContents().c_str();		}
	
	std::string& getContents() const;

protected :
	std::string mValue;
	static Type mURITypeId;
	
};

#endif //__BASICTYPES_H__
