// Type.h: interface for the Type class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __TYPE_H__
#define __TYPE_H__

#include <vector>

struct IValue;

class Type  
{
public:
	typedef IValue* (*CreateValueMethod)(const char*, void*);
	
	Type() : mTypeId(-1), mIsAbstract(false) {}
	virtual ~Type() {}
	operator int() { return mTypeId; }
	
	static Type fromName(const char* name);
	const char* getName() const;
	
	static Type badType();
	bool isBad() const;

	bool isAbstract();

	Type getParent() const;
	bool isDerivedFrom(Type type) const;
	
	bool canCreateInstance() const;
	IValue* createInstance(const char *str) const;
	
	static Type registerAbstractType(Type parent, const char* name);

	static Type registerType(Type parent, const char* name,
		CreateValueMethod createMethod=0, void *data=0);
	
private:
	int mTypeId;
	bool mIsAbstract;
	
	// Registry information of types
	struct TypeData {
		char* name;
		CreateValueMethod createMethod;
		void* data;
		int parent;
	};
	static std::vector<TypeData> mTypes;
	
	static TypeData& typeData(int typeId);
	
};

#endif //__TYPE_H__