//modified by Jing Yang after 7/6/2001
#ifndef CDimReducer_H
#define CDimReducer_H

#include <xmdv/cluster.h>
#include <xmdv/InterRing/CInterRingDimClusterTree.h>
#include <xmdv/InterRing/CInterRingDimCluster.h>

enum STATUS{ 
	NORMAL,
	CREATE,
	DISTORT, 
	MODIFY, 
	SELECT,
	CUSTOMIZE,
	APPLY,
};


enum GETDATA_STRATEGY { 
	USE_CLUSTERS, 
	USE_ORIGINALDATA
};

enum BUILDTREE_STRATEGY { 
	REVERSE_DATACLUSTERING, 
	CALCULATE_CORRELATION
};

enum AXISCOLOR_STRATEGY { 
	UNIFORM_COLOR, 
	NODE_COLOR
};


//CDimReducer: class responsible for dimension reduction
class CDimReducer
{
private:
    //strategy of getting data
    GETDATA_STRATEGY data_strategy;
    //strategy of building dimension cluster tree
    BUILDTREE_STRATEGY tree_strategy;
    //strategy of assigning axis color
    AXISCOLOR_STRATEGY axis_color_strategy;

protected:
    //data: store the data for building dimension cluster tree. data[i][j]: i: data item id; j: dimension
    double** data_array;
    int** entries_array;
    //data_num: number of data items in data_array
    int data_num;
    double *data_min;
    double *data_max;
    CInterRingDimClusterTree *dim_tree;
    CLinkList *representative_dim_list;

public:
    STATUS status;
    ARR<double> dim_max, dim_min;
	double extent_scaling_fac;
	double density;
	CInterRingDimCluster* current_node;

public:
	CDimReducer();
	~CDimReducer();

    CInterRingDimClusterTree* GetDimClusterTree();
    void AddRepresentativeDim(CInterRingDimCluster* current_node);
    void DeleteRepresentativeDim(CInterRingDimCluster* current_node);
    int  SetStatus(const char *type);
    void MapData( ARR<double> all_data, ARR<double> &data);
    void MapData( ARR<int> all_data, ARR<int> &data);
    void MapData( ARR<NominalField*> all_nominal_fields, ARR<NominalField*> &nominal_fields);
    void MapData( ARR<char *> all_names, ARR<char *> &names);
    void MapDataWithDimClusterExtends( ARR<double> all_data, ARR<double> &mean, ARR<double> &max, ARR<double> &min);

    void ExtendNodeInRepresentativeDimList(int N);
    void ShinkNodeInRepresentativeDimList(int N);
    void ResetRepresentativeDimList();
    
    bool IsLeafDim(int N);
    double GetDimDisagreement(int N);
    int GetEntries(int N);
    RGBt GetClusterColor(int N);

    void SizeStructureSelectNode(int threshold, bool mark);
//    void RadiusStructureSelectNode(double threshold, bool mark);

    void SetAxisColorStrategy(const char* arg);
    void ResetAxisColor();

    void GetSpacingDistances( double *distances );

protected:
    bool GetData();
    bool GetDataFromClusters();
    void CollectClustersOnContour(double contour_radius, Cluster node, CLinkList* data_linkList, CLinkList* entries_linkList);
    void RefineData();
    void SaveDataToFile(const char * fname);
    bool BuildDimClusterTree();
    void SaveMinMax();

};

extern bool CreateDimClusterFile(const char * data_fname, const char* result_fname);

#endif

