// developed by debbie 3/2003
#include <cmath>
#include <iostream>
#include <cstdlib>
#include <ctime>

using namespace std;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////helper function for calculating the distances between dimensions//////////////////////////////////

double calc_scale( double num, double **data, int data_num, int dimension)
{
	double scale;
	double max = num, min = num;

	////////////////////////////////////////get the max and min value of one dimension///////////////////////////////////////////
	for(int count = 0; count < data_num; count++ )
	{
		if(data[count][dimension] > max)
			max = data[count][dimension];
		if(data[count][dimension] < min)
			min = data[count][dimension];
	}

	if(max == min)
		scale = 1;
	else
		scale = (num - min)/(max - min);

	return scale;

}

////////////////////////////////////////////get the similarity between two dimensions////////////////////////////////////////////
double simi_func(double **data, int data_num, int dim_1, int dim_2)
{
	double scale_1, scale_2, total=0;
	int count = 0;

	//////////////////////calculate distance between dimension 1 and dimension 2//////////////////////////////////////
	for ( count = 0; count < data_num; count++ )
	{
		scale_1 = calc_scale(data[count][dim_1],data,data_num,dim_1);
		scale_2 = calc_scale(data[count][dim_2],data,data_num,dim_2);

		total += pow(scale_1-scale_2,2);
	}
	
	total = sqrt(total);

	return total;
}

///////////////////////////////////////calculate the sum of distances between neighboring dimensions//////////////////////////////
/////////array gives the sequence of dimensions

double sim_total(double **similarity_matrix, int dim_num, int *array)
{
	double total = 0;
	
	for(int count = 0; count < dim_num-1; count++ )
	{
		total += similarity_matrix[array[count]][array[count+1]];
	}

	return total;
}

///////////////////////////////////////helper function , swap two elements in an array//////////////////////////////////////////
void swap_element(int *array, int index_1, int index_2)
{
	int temp;
	temp = array[index_1];
	array[index_1] = array[index_2];
	array[index_2] = temp;
	return;
}

//////////////////////////////////////recursive function for the optimal ordering/////////////////////////////////////////////////

void recur_order(double **similarity, int dim_num, int *temp_array, int *order_array, int start)
{
	int count;
	if(start == dim_num)  //the exit condition for the recurrence
	{
		if ( sim_total(similarity, dim_num, order_array) > sim_total(similarity, dim_num, temp_array))
			for(count = 0; count < dim_num; count++ )
				order_array[count] = temp_array[count];
		
		for(count = 0; count< dim_num; count++)
			cout<<order_array[count]<<"             "<<temp_array[count]<<endl;

		cout<<sim_total(similarity, dim_num, order_array)<<"  "<<sim_total(similarity, dim_num, temp_array)<<endl;
				
	}
		
	else
	{
		for( count = start; count < dim_num; count++ )
		{
			swap_element(temp_array, count, start);
			recur_order(similarity, dim_num, temp_array, order_array, start+1);
			swap_element(temp_array, count, start);
		}

	}

	return;
}

///////////////////////////////////////////////function for optimal ordering///////////////////////////////////////

void optimal_reorder(double **similarity, int dim_num, int *order_array)
{
	int *temp_array = new int[dim_num];

	for(int count = 0; count < dim_num; count++ )  // initialize the order array
	{
		order_array[count] = count;
		temp_array[count] = count;
	}

	recur_order(similarity, dim_num, temp_array, order_array, 0);
	delete temp_array;
}

/////////////////////////////////////////////function for random ordering/////////////////////////////////////////
void random_reorder(double **similarity, int dim_num, int *order_array)
{
	int *temp_array = new int[dim_num];
	
	int count =0;
	for(count = 0; count < dim_num; count++ )  // initialize the order array
	{
		order_array[count] = count;
		temp_array[count] = count;
	}

	int dim_1, dim_2; 
	
	srand( (unsigned)time( NULL ));

	for(int times = 0; times < 10; times++) /////////////10 can be substituted by other meaningful numbers
	{
		dim_1 = rand()%dim_num;
		dim_2 = rand()%dim_num;
	    cout<<"dim_1 "<<dim_1<<"          "<<"dim_2 "<<dim_2<<endl;
		swap_element(temp_array, dim_1, dim_2);
		
		if ( sim_total(similarity, dim_num, order_array) > sim_total(similarity, dim_num, temp_array))
			for(count = 0; count < dim_num; count++ )
				order_array[count] = temp_array[count];
			
		for(count = 0; count< dim_num; count++)
			cout<<order_array[count]<<"             "<<temp_array[count]<<endl;

		cout<<sim_total(similarity, dim_num, order_array)<<"       "<<sim_total(similarity,dim_num,temp_array)<<endl;

	}
	delete temp_array;
}
				


////////////////////////////////////////////////reorder using similarity method////////////////////////////////////
/////////choice: 1,optimal reorder; 2,random reorder; 3,artificial ants reorder////////////////////////////////////

bool similarity_reorder(int n, int m, double **data, double *order_para, int choice) 
//n: data points, m: dimension
{
	int row, column;
	double sum_of_similarity;    // for spacing purpose
	int *order_array = new int[m];

	double **simi_matrix = new double *[m]; // Similarity matrix of the n data points
	int count = 0;
	for(count = 0; count < m; count++ )
		simi_matrix[count] = new double[m];
	
	/////////////////get the similarity matrix of the data set///////////////////////////
	for(row = 0 ; row < m; row++)
		for(column = 0; column < m; column++)
		{
			simi_matrix[row][column] = simi_matrix[column][row] = simi_func(data, n, row, column);
			cout<<simi_matrix[row][column]<<endl;
		}

	//////////////////////reorder the dimensions//////////////////////////////////////////

	if( choice == 1 )
	{
		optimal_reorder(simi_matrix, m, order_array);
	}
	else if(choice == 2)
		random_reorder(simi_matrix, m, order_array);
	else 
		return false;

	sum_of_similarity = order_para[order_array[m-1]] = simi_matrix[order_array[m-1]][order_array[0]]; 
	// the order parameter of the last dimension and the first dimension in the ordered array
	for(count = m-2; count >= 0; count--)
		sum_of_similarity = order_para[order_array[count]] = sum_of_similarity + simi_matrix[order_array[count+1]][order_array[count]];
		// get the order parameters: the front dimensions have larger order_para
	
	for( count = 0; count < m; count++ )
		delete simi_matrix[count];
	delete simi_matrix;
	delete order_array;
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*
void main()
{
	double **sample = new double *[3];
	for ( int count = 0; count < 3; count++ )
		sample[count] = new double[5];

//	sample[5][3] = {{1,5,2},{4,5,7},{2,8,0},{3,4,10},{4,3,1}};

	sample[0][0] = 1;
	sample[0][1] = 3;
	sample[0][2] = 2;
	sample[0][3] = 2;
	sample[0][4] = 6;
	sample[1][0] = 3;
	sample[1][1] = 3;
	sample[1][2] = 0;
	sample[1][3] = 4;
	sample[1][4] = 4;
	sample[2][0] = 8;
	sample[2][1] = 5;
    sample[2][2] = 5;
	sample[2][3] = 9;
	sample[2][4] = 6;

*/
/*	sample[0][0] = 1;
	sample[0][1] = 3;
	sample[0][2] = 2;
	sample[1][0] = 2;
	sample[1][1] = 6;
	sample[1][2] = 3;
	sample[2][0] = 3;
	sample[2][1] = 0;
	sample[2][2] = 4;
	sample[3][0] = 4;
	sample[3][1] = 8;
	sample[3][2] = 5;
    sample[4][0] = 5;
	sample[4][1] = 9;
	sample[4][2] = 6;
	*/

/*	for(int i = 0; i < 4; i++)
		for(int j = 0; j < 4; j++)
			sample[i][j] = i*j;
*/	
/*	double *order = new double[5];

	similarity_reorder(3,5,sample,order,2);
	for(count = 0; count < 5; count ++)
		cout<<order[count]<< "  ";
	
	delete order;
	for( count = 0; count < 3; count++ )
		delete sample[count];
	delete sample;
}*/
